@@ -1,15 +1,15 @@
 fun.load.widals.fill <-
 function() {
     
-    
-    xenvr <- as.environment(1)
-    
+    lags <- lags
+    run.parallel <- run.parallel
+
     
     if( run.parallel ) {
         
         sfExport("Z", "Hs", "Ht", "Hst.ls", "locs", "lags", "b.lag", "cv", "rm.ndx", "train.rng", "test.rng", "xgeodesic", "ltco", "stnd.d", "Z.na")
         
-        suppressWarnings(sfLibrary(widals))
+        suppressWarnings(sfLibrary("widals", character.only=TRUE))
         
     }
     
@@ -19,14 +19,30 @@ function() {
     } else {
         p.ndx.ls <- list( c(1,2), c(3,4,5) )
     }
-    assign( "p.ndx.ls", p.ndx.ls, pos=xenvr )
+    ## assign( "p.ndx.ls", p.ndx.ls, pos=globalenv() )
+    p.ndx.ls <<- p.ndx.ls
     
     f.d <- list( dlog.norm, dlog.norm, dlog.norm, dlog.norm, dlog.norm )
-    assign( "f.d", f.d, pos=xenvr )
-    
+    ## assign( "f.d", f.d, pos=globalenv() )
+    f.d <<- f.d
     
     FUN.MH <- function(jj, GP.mx, X) {
         
+        rm.ndx <- rm.ndx
+        Hs <- Hs
+        Ht <- Ht
+        Hst.ls <- Hst.ls
+        locs <- locs
+        lags <- lags
+        b.lag <- b.lag
+        cv <- cv
+        xgeodesic <- xgeodesic
+        stnd.d <- stnd.d
+        ltco <- ltco
+        Z <- Z
+        train.rng <- train.rng
+        Z.na <- Z.na
+        
         Z.wid <- widals.snow(jj, rm.ndx=rm.ndx, Z=X$Z.fill, Hs=Hs, Ht=Ht, Hst.ls=Hst.ls, locs=locs, lags=lags, b.lag=b.lag,
         cv=cv, geodesic=xgeodesic, wrap.around=NULL, GP.mx=GP.mx, stnd.d=stnd.d, ltco=ltco )
         
@@ -38,12 +54,19 @@ function() {
         
         return( our.cost )
     }
-    assign( "FUN.MH", FUN.MH, pos=xenvr )
+    ## assign( "FUN.MH", FUN.MH, pos=globalenv() )
+    FUN.MH <<- FUN.MH
     
     #FUN.GP <- NULL
     
     
     FUN.GP <- function(GP.mx) {
+        
+        rho.upper.limit <- rho.upper.limit
+        rgr.lower.limit <- rgr.lower.limit
+        d.alpha.lower.limit <- d.alpha.lower.limit
+        
+        
         GP.mx[ GP.mx[ , 1] > rho.upper.limit, 1 ] <- rho.upper.limit
         GP.mx[ GP.mx[ , 2] < rgr.lower.limit, 2 ] <- rgr.lower.limit
         GP.mx[ GP.mx[ , 3] < d.alpha.lower.limit, 3 ] <- d.alpha.lower.limit
@@ -53,17 +76,36 @@ function() {
         return(GP.mx)
         
     }
-    assign( "FUN.GP", FUN.GP, pos=xenvr )
-    
+    ## assign( "FUN.GP", FUN.GP, pos=globalenv() )
+    FUN.GP <<- FUN.GP
     
     FUN.I <- function(envmh, X) {
+        
+        rm.ndx <- rm.ndx
+        
+        Hs <- Hs
+        Ht <- Ht
+        Hst.ls <- Hst.ls
+        locs <- locs
+        lags <- lags
+        
+        b.lag <- b.lag
+        cv <- cv
+        xgeodesic <- xgeodesic
+        stnd.d <- stnd.d
+        ltco <- ltco
+        Z.na <- Z.na
+        
+
+        
         GP.mx <- matrix(envmh$GP, 1, length(envmh$GP))
         Z.wid <- widals.snow(1, rm.ndx=rm.ndx, Z=X$Z.fill, Hs=Hs, Ht=Ht, Hst.ls=Hst.ls, locs=locs, lags=lags, b.lag=b.lag,
         cv=cv, geodesic=xgeodesic, wrap.around=NULL, GP.mx=GP.mx, stnd.d=stnd.d, ltco=ltco )
         
         ### if( min(Z, na.rm=TRUE) >= 0 ) { Z.wid[ Z.wid < 0 ] <- 0 } ############ DZ EDIT
         
-        assign( "Z.wid", Z.wid, envir=xenvr )
+        ## assign( "Z.wid", Z.wid, envir=globalenv() )
+        Z.wid <<- Z.wid
         Z.wid <- Z.clean.up(Z.wid)
         
         
@@ -74,20 +116,27 @@ function() {
         cat( "Improvement ---> ", envmh$current.best, " ---- " , envmh$GP, "\n" )
         return(X)
     }
-    assign( "FUN.I", FUN.I, pos=xenvr )
-    
+    ## assign( "FUN.I", FUN.I, pos=globalenv() )
+    FUN.I <<- FUN.I
     
     FUN.EXIT <- function(envmh, X) {
         
         our.cost <- envmh$current.best
-        assign( "our.cost", our.cost, pos=xenvr )
+        ## assign( "our.cost", our.cost, pos=globalenv() )
+        ## assign( "Z.fill", X$Z.fill, envir=globalenv() )
+        ## assign( "GP", envmh$GP, pos=globalenv() )
+        
+        our.cost <<- our.cost
+        Z.fill <- X$Z.fill
+        Z.fill <<- Z.fill
+        GP <- envmh$GP
+        GP <<- GP
         
-        assign( "Z.fill", X$Z.fill, envir=xenvr )
-        assign( "GP", envmh$GP, pos=xenvr )
         cat( paste( "GP <- c(", paste(format(envmh$GP,digits=5), collapse=", "), ") ### ", format(our.cost, width=6), "\n", sep="" ) )
         
         
     }
-    assign( "FUN.EXIT", FUN.EXIT, pos=xenvr )
+    ## assign( "FUN.EXIT", FUN.EXIT, pos=globalenv() )
+    FUN.EXIT <<- FUN.EXIT
     
 }
